enum Attrs {A_REVERSE,A_NORMAL,A_XOR,A_SHADED,A_REPLACE,A_OR,A_AND,A_THICK1,
            A_SHADE_V,A_SHADE_H,A_SHADE_NS,A_SHADE_PS};

/*---------------------------------------------------------------------------*/
/* speedy pixel access routines                                              */
/*---------------------------------------------------------------------------*/
#define PIXOFFSET(x,y)  ((y<<5)-(y<<1)+(x>>3))
#define PIXADDR(p,x,y)  (((unsigned char*)(p))+PIXOFFSET(x,y))
#define PIXMASK(x)      ((unsigned char)(0x80 >> ((x)&7)))

#define SETPIX(a,m)   (*(a) |= (m)) // uses precalculated values now
#define CLRPIX(a,m)   (*(a) &= ~(m))
#define XORPIX(a,m)   (*(a) ^= (m))
#define PIXUP(a)      ((a)-=30)
#define PIXDOWN(a)    ((a)+=30)
#define PIXLEFT(a,m)   asm("rol.b  #1,%0;bcc.s  0f;subq.l #1,%1;0:"\
                          : "=d" (m), "=g" (a) : "0" (m), "1" (a))
#define PIXRIGHT(a,m)  asm("ror.b  #1,%0;bcc.s  0f;addq.l #1,%1;0:"\
                          : "=d" (m), "=g" (a) : "0" (m), "1" (a))


/*---------------------------------------------------------------------------*/
/* useful macros                                                             */
/*---------------------------------------------------------------------------*/
#define ABS(a)          (((a)<0) ? -(a): (a))
#define SPECIAL_SGN(a)  (((a)<0) ? 0 : 1)


/*===========================================================================*/
/* draws a line from (x1,y1) to (x2,y2) in given plane                       */
/*                                                                           */
/* valid modes: A_REPLACE,A_NORMAL,A_REVERSE,A_XOR                           */
/*===========================================================================*/
void FastDrawLine(unsigned char* plane,
                  short x1,short y1,
                  short x2,short y2,
                  short mode)
{
    short d,x,y,ax,ay,sx,sy,dx,dy;

    unsigned char *pixaddr;
    unsigned char pixmask;

    dx = x2 - x1;
    ax = ABS(dx)<<1;
    sx = SPECIAL_SGN(dx);

    dy = y2 - y1;
    ay = ABS(dy)<<1;
    sy = SPECIAL_SGN(dy);

    x = x1;
    y = y1;

    pixaddr=PIXADDR(plane, x, y);
    pixmask=PIXMASK(x);

    //-------------------------------------------------------------------------
    // for speedup reasons the different modes are handled by own blocks.
    // The only difference is the usage of SETPIX/CLRPIX or XORPIX
    //-------------------------------------------------------------------------

    // sets each pixel of the line ...

    if (mode == A_REPLACE || mode == A_NORMAL) {
        if (ax>ay) {           /* x dominant */
            d = ay - (ax>>1);
            do {
                SETPIX(pixaddr,pixmask);
                if (x==x2) return;
                if (d>=0) {
                    if(sy) PIXDOWN(pixaddr);
                    else   PIXUP(pixaddr);
                    d -= ax;
                }
                if (sx) {
                    PIXRIGHT(pixaddr, pixmask);
                    x++;
                }
                else {
                    PIXLEFT(pixaddr, pixmask);
                    x--;
                }
                d += ay;
            } while(1);
        }
        else {                 /* y dominant */
            d = ax - (ay>>1);
            do {
                SETPIX(pixaddr,pixmask);
                if (y==y2) return;
                if (d>=0) {
                    if (sx) PIXRIGHT(pixaddr, pixmask);
                    else    PIXLEFT(pixaddr, pixmask);
                    d -= ay;
                }
                if (sy) {
                    PIXDOWN(pixaddr);
                    y++;
                }
                else {
                    PIXUP(pixaddr);
                    y--;
                }
                d += ax;
            } while(1);
        }
    }

    // clears each pixel of the line ...
    else if (mode == A_REVERSE) {
        if (ax>ay) {           /* x dominant */
            d = ay - (ax>>1);
            do {
                CLRPIX(pixaddr,pixmask);
                if (x==x2) return;
                if (d>=0) {
                    if (sy) PIXDOWN(pixaddr);
                    else    PIXUP(pixaddr);
                    d -= ax;
                }
                if (sx) {PIXRIGHT(pixaddr, pixmask);x++;}
                else    {PIXLEFT(pixaddr, pixmask);x--;}
                d += ay;
            } while(1);
        }
        else {                 /* y dominant */
            d = ax - (ay>>1);
            do {
                CLRPIX(pixaddr,pixmask);
                if (y==y2) return;
                if (d>=0) {
                    if (sx) PIXRIGHT(pixaddr, pixmask);
                    else    PIXLEFT(pixaddr, pixmask);
                    d -= ay;
                }
                if(sy) {PIXDOWN(pixaddr);y++;}
                else   {PIXUP(pixaddr);y--;}
                d += ax;
            } while(1);
        }
    }

    // XORs each pixel of the line ...

    else if (mode == A_XOR) {
        if (ax>ay) {           /* x dominant */
            d = ay - (ax>>1);
            do {
                XORPIX(pixaddr,pixmask);
                if (x==x2) return;
                if (d>=0) {
                    if (sy) PIXDOWN(pixaddr);
                    else    PIXUP(pixaddr);
                    d -= ax;
                }
                if (sx) {PIXRIGHT(pixaddr, pixmask);x++;}
                else    {PIXLEFT(pixaddr, pixmask);x--;}
                d += ay;
            } while(1);
        }
        else {                 /* y dominant */
            d = ax - (ay>>1);
            do {
                XORPIX(pixaddr,pixmask);
                if (y==y2) return;
                if (d>=0) {
                    if (sx) PIXRIGHT(pixaddr, pixmask);
                    else    PIXLEFT(pixaddr, pixmask);
                    d -= ay;
                }
                if (sy) {PIXDOWN(pixaddr);y++;}
                else    {PIXUP(pixaddr);y--;}
                d += ax;
            } while(1);
        }
    }

    // NOTE: if the mode is not within A_REPLACE/A_REVERSE/A_XOR nothing is drawn !!
}
