// macro which exchanges the content of two variables using the ASM exg instruction
#define EXT_XCHG(a,b)    asm volatile ("exg %0,%1" : "=d" (a), "=d" (b) : "0" (a), "1" (b) : "cc")

// checks for collision between 2 sprites of width 16
// returns 1 if a collision is detected
short TestCollide16(short x0,short y0,short x1,short y1,
                    short height,unsigned short* data0, unsigned short* data1)
{
    short dx,dy,i;

    // sprite 0 is left of sprite 1 ...
    if (x0<x1) {
        dx = x1-x0;
    }
    else {
        // if sprite 0 is right of sprite 1 just exchange
        // all necessary variables so we can still assume
        // sprite 0 is left of sprite 1 ...
        dx = x0-x1;
        EXT_XCHG(data0,data1);
        EXT_XCHG(y0,y1);
    }

    if (dx>=16) return 0; // cannot overlap in this case (x difference too large)

    if (y0<y1) {
        dy = y1-y0;
        data0+=dy;
    }
    else {
        dy = y0-y1;
        data1+=dy;
    }

    for (i=dy;i<height;i++,data0++,data1++) {
        // Let's shift the data of sprite 0 to the left and AND it with
        // the data of sprite 1. If this results in at least one bit set,
        // we have a collision
        if (((unsigned long)*data1) & (((unsigned long)*data0) << dx)) return 1;
    }

    // if we come here we have found no collision!
    return 0;
}
